@@ -22,8 +22,11 @@
    Boston, MA 02110-1301, USA.
 */
 
+#define _XOPEN_SOURCE
 #include "kpty_p.h"
 
+#include <QProcess>
+
 #ifdef __sgi
 #define __svr4__
 #endif
@@ -71,10 +74,27 @@
 # include <util.h>
 #endif
 
-#if HAVE_UTEMPTER
-extern "C" {
-# include <utempter.h>
-}
+#ifdef UTEMPTER_PATH
+// utempter uses 'add' and 'del' whereas ulog-helper uses 'login' and 'logout'
+#  ifndef UTEMPTER_ULOG
+#    define UTEMPTER_ADD "add"
+#    define UTEMPTER_DEL "del"
+#  else
+#    define UTEMPTER_ADD "login"
+#    define UTEMPTER_DEL "logout"
+#  endif
+class UtemptProcess : public QProcess
+{
+public:
+    virtual void setupChildProcess() override
+    {
+        // These are the file descriptors the utempter helper wants
+        dup2(cmdFd, 0);
+        dup2(cmdFd, 1);
+        dup2(cmdFd, 3);
+    }
+    int cmdFd;
+};
 #else
 # include <utmp.h>
 # if HAVE_UTMPX
@@ -149,6 +169,9 @@ extern "C" {
 KPtyPrivate::KPtyPrivate(KPty *parent) :
     masterFd(-1), slaveFd(-1), ownMaster(true), q_ptr(parent)
 {
+#ifdef UTEMPTER_PATH
+    utempterPath = QStringLiteral(UTEMPTER_PATH);
+#endif
 }
 
 KPtyPrivate::~KPtyPrivate()
@@ -206,7 +229,7 @@ bool KPty::open()
 #if HAVE_OPENPTY
 
     char ptsn[PATH_MAX];
-    if (::openpty(&d->masterFd, &d->slaveFd, ptsn, 0, 0)) {
+    if (::openpty(&d->masterFd, &d->slaveFd, ptsn, nullptr, nullptr)) {
         d->masterFd = -1;
         d->slaveFd = -1;
         qWarning() << "Can't open a pseudo teletype";
@@ -493,11 +516,22 @@ void KPty::setCTty()
 
 void KPty::login(const char *user, const char *remotehost)
 {
-#if HAVE_UTEMPTER
+#ifdef UTEMPTER_PATH
     Q_D(KPty);
 
-    addToUtmp(d->ttyName.constData(), remotehost, d->masterFd);
     Q_UNUSED(user);
+
+    // Emulating libutempter version 1.1.6
+    if (!d->utempterPath.isEmpty()) {
+        UtemptProcess utemptProcess;
+        utemptProcess.cmdFd = d->masterFd;
+        utemptProcess.setProgram(d->utempterPath);
+        utemptProcess.setArguments(QStringList() << QStringLiteral(UTEMPTER_ADD) << QString::fromLocal8Bit(remotehost));
+        utemptProcess.setProcessChannelMode(QProcess::ForwardedChannels);
+        utemptProcess.start();
+        utemptProcess.waitForFinished();
+    }
+
 #else
 # if HAVE_LOGINX
     struct utmpx l_struct;
@@ -573,10 +607,20 @@ void KPty::login(const char *user, const char *remotehost)
 
 void KPty::logout()
 {
-#if HAVE_UTEMPTER
+#ifdef UTEMPTER_PATH
     Q_D(KPty);
 
-    removeLineFromUtmp(d->ttyName.constData(), d->masterFd);
+    // Emulating libutempter version 1.1.6
+    if (!d->utempterPath.isEmpty()) {
+        UtemptProcess utemptProcess;
+        utemptProcess.cmdFd = d->masterFd;
+        utemptProcess.setProgram(d->utempterPath);
+        utemptProcess.setArguments(QStringList(QStringLiteral(UTEMPTER_DEL)));
+        utemptProcess.setProcessChannelMode(QProcess::ForwardedChannels);
+        utemptProcess.start();
+        utemptProcess.waitForFinished();
+    }
+
 #else
     Q_D(KPty);
 
